/**
 * @file
 * Behaviors for the form math custom element.
 */

((Drupal, math) => {
  Drupal.behaviors.formCalculationElement = {
    attach: () => {
      const FIELD_SEPARATOR = '|';
      const EVALUATE_FIELD_SEPARATOR = ' ';
      const previousValue = new WeakMap();

      const numberFormat = (value, format = 'none', currency = 'USD') => {
        let formattedNumber = 0;
        const lang = drupalSettings.path.currentLanguage || 'en';

        switch (format) {
          case 'currency':
            try {
              formattedNumber = new Intl.NumberFormat(lang, {
                style: format,
                currency,
              }).format(value);
            } catch (e) {
              // eslint-disable-next-line no-console
              console.error(
                `Failed to format currency: Visit the element settings and check if the currency value is correct. Fail ${e}`,
              );
              formattedNumber = new Intl.NumberFormat().format(value);
            }
            break;
          case 'numeric':
            formattedNumber = new Intl.NumberFormat().format(value);
            break;
          default:
            formattedNumber = value;
        }
        return formattedNumber;
      };

      const updateResult = (el, value) => {
        if (el.tagName === 'INPUT') {
          el.value = numberFormat(
            value,
            el.dataset.inputmask,
            el.dataset.inputmaskCurrency,
          );
          el.dispatchEvent(new Event('change'));
          return;
        }
        const markupEl = el.querySelector('.form-math-markup-element');
        if (markupEl === null) {
          return;
        }

        const expressionEvaluated = markupEl
          .querySelector('.js-math-result-base')
          .innerHTML.replace(
            '[RESULT]',
            numberFormat(
              value,
              markupEl.dataset.inputmask,
              markupEl.dataset.inputmaskCurrency,
            ),
          );

        markupEl.querySelector('.js-math-result-input').value =
          expressionEvaluated;
        markupEl.querySelector('.js-math-result').innerHTML =
          expressionEvaluated;
        if (value !== '') {
          markupEl.classList.add('form-math-markup-element-calculated');
        } else {
          markupEl.classList.remove('form-math-markup-element-calculated');
        }
      };

      const calc = (fieldElement, formId) => {
        const fields = fieldElement.dataset.evaluateFields.split(
          EVALUATE_FIELD_SEPARATOR,
        );
        const expression = fieldElement.dataset.evaluateExpression;
        const deepFields = JSON.parse(
          fieldElement.dataset.evaluateDeepFields || '{}',
        );
        const fieldEvaluateDecimals =
          fieldElement.dataset.evaluateDecimals || null;

        const buildExpression = (exp, field) => {
          const [fieldName, defaultValue] = field.split(FIELD_SEPARATOR);
          // Attempt to pick up the checked input first (e.g. radio buttons).
          let currentElement = document.querySelector(
            `form#${formId} input[name="${fieldName}"]:checked`,
          );
          if (!currentElement) {
            currentElement = document.querySelector(
              `form#${formId} input[name="${fieldName}"], form#${formId} select[name="${fieldName}"]`,
            );
          }
          let currentFieldValue;
          // If the field is not found, check if it is a deep field.
          if (currentElement === null && deepFields[fieldName] !== undefined) {
            currentFieldValue = deepFields[fieldName];
          } else {
            currentFieldValue = currentElement.value;
            if (currentFieldValue === '' && defaultValue !== undefined) {
              currentFieldValue = defaultValue;
            }
          }
          let fieldExp = fieldName;
          if (defaultValue !== undefined) {
            fieldExp += `|${defaultValue}`;
          }
          if (currentFieldValue !== '') {
            return exp.replaceAll(`:${fieldExp}`, currentFieldValue);
          }
          return exp;
        };

        const handleFieldChange = (e) => {
          // If the value was clear. Rollback the element to the initial state.
          if (
            e.target.value === '' &&
            previousValue.has(e.target) &&
            previousValue.get(e.target) !== ''
          ) {
            updateResult(fieldElement, '');
            const markupContainers = document.querySelectorAll(
              `form#${formId} div[data-evaluate-fields]`,
            );
            markupContainers.forEach((container) => {
              // Check if the container markup element depends on the field that was cleared.
              if (
                container.dataset.evaluateFields
                  .split(EVALUATE_FIELD_SEPARATOR)
                  .includes(e.target.name)
              ) {
                container
                  .querySelector('.form-math-markup-element')
                  .classList.remove('form-math-markup-element-calculated');
              }
            });
          }

          const convertedExpression = fields.reduce(
            buildExpression,
            expression,
          );
          // Check if remaining field to be populated.
          if (convertedExpression.split(':').length === 1) {
            let expressionResult = math.evaluate(convertedExpression);
            if (fieldEvaluateDecimals !== null) {
              expressionResult = parseFloat(expressionResult).toFixed(
                fieldEvaluateDecimals,
              );
            }
            updateResult(fieldElement, expressionResult);
          }
          previousValue.set(e.target, e.target.value);
        };

        const selectorFormContext = `form#${formId}`;
        fields.forEach((fieldExp) => {
          const [name] = fieldExp.split(FIELD_SEPARATOR);
          const elementSelector = `${selectorFormContext} input[name="${name}"], ${selectorFormContext} select[name="${name}"]`;
          // Listen to all input instances.
          document.querySelectorAll(elementSelector).forEach((element) => {
            if (element) {
              element.removeEventListener('change', handleFieldChange);
              element.addEventListener('change', handleFieldChange);
            }
          });
        });
      };

      const triggerOpenedForm = (fields, formId) => {
        const changeEvent = new Event('change');
        fields.forEach((field) => {
          const [fieldName, defaultValue] = field.split(FIELD_SEPARATOR);
          const input = document.querySelector(
            `form#${formId} [name~="${fieldName}"]`,
          );
          if (defaultValue !== undefined || input.value !== '') {
            if (input) {
              input.dispatchEvent(changeEvent);
            }
          }
        });
      };

      const processFormCalculationElements = (el, formId) => {
        const data = el.dataset;
        if (
          data.evaluateFields === undefined ||
          data.evaluateExpression === undefined
        ) {
          return;
        }

        try {
          calc(el, formId);
          triggerOpenedForm(
            data.evaluateFields.split(EVALUATE_FIELD_SEPARATOR),
            formId,
          );
        } catch (e) {
          // eslint-disable-next-line no-console
          console.error(`Failed to calculate ${e}`);
        }
      };

      let forms = Array.from(document.querySelectorAll('form'));
      forms = forms.filter(
        (form) =>
          form.querySelector('.js-form-math-element') &&
          !form.classList.contains('built'),
      );
      if (forms.length === 0) {
        return;
      }

      forms.forEach((form) => {
        form
          .querySelectorAll('.js-form-math-element')
          .forEach((el) => processFormCalculationElements(el, form.id));
      });
    },
  };
})(Drupal, math);
